@@ -3,7 +3,7 @@ import itertools
 import pickle
 import rpy2
 import rpy2.rinterface as rinterface
-import sys, os, subprocess, time, tempfile, signal
+import sys, os, subprocess, time, tempfile, io, signal, gc
 
 IS_PYTHON3 = sys.version_info[0] == 3
 
@@ -30,19 +30,35 @@ class EmbeddedRTestCase(unittest.TestCase):
 
 
     def testConsolePrint(self):
-        tmp_file = tempfile.NamedTemporaryFile()
-        stdout = sys.stdout
-        sys.stdout = tmp_file
-        try:
-            rinterface.consolePrint('haha')
-        except Exception, e:
+        if sys.version_info[0] == 3:
+            tmp_file = io.StringIO()
+            stdout = sys.stdout
+            sys.stdout = tmp_file
+            try:
+                rinterface.consolePrint('haha')
+            except Exception, e:
+                sys.stdout = stdout
+                raise e
             sys.stdout = stdout
-            raise e
-        sys.stdout = stdout
-        tmp_file.flush()
-        tmp_file.seek(0)
-        self.assertEqual('haha', ''.join(s.decode() for s in tmp_file))
-        tmp_file.close()
+            tmp_file.flush()
+            tmp_file.seek(0)
+            self.assertEqual('haha', ''.join(s for s in tmp_file).rstrip())
+            tmp_file.close()
+        else:
+            # no need to test which Python 2, only 2.7 supported
+            tmp_file = tempfile.NamedTemporaryFile()
+            stdout = sys.stdout
+            sys.stdout = tmp_file
+            try:
+                rinterface.consolePrint('haha')
+            except Exception, e:
+                sys.stdout = stdout
+                raise e
+            sys.stdout = stdout
+            tmp_file.flush()
+            tmp_file.seek(0)
+            self.assertEqual('haha', ''.join(s.decode() for s in tmp_file))
+            tmp_file.close()
 
 
     def testCallErrorWhenEndedR(self):
@@ -107,6 +123,17 @@ class EmbeddedRTestCase(unittest.TestCase):
         res = rfun(1, 2)
         self.assertEqual(3, res[0])
 
+    def testRternalizeNamedArgs(self):
+        def f(x, y, z=None):
+            if z is None:
+                return x[0]+y[0]
+            else:
+                return z
+        rfun = rinterface.rternalize(f)
+        res = rfun(1, 2)
+        self.assertEqual(3, res[0])
+        res = rfun(1, 2, z=8)
+        self.assertEqual(8, res[0])
 
     def testExternalPython(self):
         def f(x):
@@ -170,6 +197,14 @@ except Exception%s e:
         ret_code = child_proc.poll()
         self.assertFalse(ret_code is None) # Interruption failed
 
+    def testRpyMemory(self):
+        x = rinterface.SexpVector(xrange(10), rinterface.INTSXP)
+        y = rinterface.SexpVector(xrange(10), rinterface.INTSXP)
+        x_rid = x.rid
+        self.assertTrue(x_rid in set(z[0] for z in rinterface.protected_rids()))
+        del(x)
+        gc.collect(); gc.collect()
+        self.assertFalse(x_rid in set(z[0] for z in rinterface.protected_rids()))
 
 class CallbacksTestCase(unittest.TestCase):
     def tearDown(self):
